#define DEBUGTRACE_ENABLED
#include "lasp_rtaps.h"
#include "debugtrace.hpp"
#include <mutex>

using std::cerr;
using std::endl;

bool RtAps::inCallback_threaded(const DaqData &data) {

  /* DEBUGTRACE_ENTER; */

  std::scoped_lock<std::mutex> lck(_mtx);
  dmat fltdata = data.toFloat();
  const us nchannels = fltdata.n_cols;

  if (_filterPrototype) {

    // Adjust number of filters, if necessary
    if (nchannels > _freqWeightingFilter.size()) {
      while (nchannels > _freqWeightingFilter.size()) {
        _freqWeightingFilter.emplace_back(_filterPrototype->clone());
      }

      for (auto &filter : _freqWeightingFilter) {
        filter->reset();
      }
    }

    // Apply filtering
#pragma omp parallel for
    for (us i = 0; i < nchannels; i++) {
      vd col = fltdata.col(i);
      _freqWeightingFilter.at(i)->filter(col);
      fltdata.col(i) = col;
    }
  } // End of if(_filterPrototype)

  std::optional<arma::cx_cube> res = _ps.compute(fltdata);

  if (res.has_value()) {
    /* DEBUGTRACE_PRINT("Data ready!"); */
    _latest_est = std::make_unique<cube>(std::move(res.value()));
  }

  return true;
}
void RtAps::reset(const Daq *daq) { // Explicitly say
                                    // to GCC that
                                    // the argument is
                                    // not used.

  DEBUGTRACE_ENTER;
  std::scoped_lock<std::mutex> lck(_mtx);
  _ps.reset();
  _latest_est.reset();
}

std::unique_ptr<cube> RtAps::getCurrentValue() {

  /* DEBUGTRACE_ENTER; */
  std::scoped_lock<std::mutex> lck(_mtx);

  return std::move(_latest_est);
}
