/* #define DEBUGTRACE_ENABLED */
#include "debugtrace.hpp"
#include "lasp_siggen.h"
#include "lasp_mathtypes.h"
#include <cassert>
#include <type_traits>
using std::cerr;
using std::endl;

inline d level_amp(d level_dB) { return pow(10, level_dB / 20); }

using mutexlock = std::scoped_lock<std::mutex>;

vd Siggen::genSignal(const us nframes) {

  DEBUGTRACE_ENTER;
  mutexlock lck(_mtx);

  DEBUGTRACE_PRINT(nframes);
  vd signal(nframes, arma::fill::value(_dc_offset));

  if (!_muted) {
    vd signal_dynamic = _level_linear * genSignalUnscaled(nframes);
    if (_filter) {
      _filter->filter(signal_dynamic);
    }
    signal += signal_dynamic;
  }

  return signal;
}
void Siggen::setFilter(std::shared_ptr<Filter> filter) {
  DEBUGTRACE_ENTER;
  mutexlock lck(_mtx);
  _filter = filter;
}
void Siggen::setDCOffset(const d offset) {
  DEBUGTRACE_ENTER;
  mutexlock lck(_mtx);
  _dc_offset = offset;
}
void Siggen::setLevel(const d level, bool dB) {
  DEBUGTRACE_ENTER;
  mutexlock lck(_mtx);
  _level_linear = dB ? level_amp(level) : level;
}
void Siggen::reset(const d newFs) {
  DEBUGTRACE_ENTER;
  mutexlock lck(_mtx);
  _fs = newFs;
  resetImpl();
}
