/* #define DEBUGTRACE_ENABLED */
#include "lasp_threadedindatahandler.h"
#include "debugtrace.hpp"
#include "lasp_thread.h"
#include <future>
#include <thread>

using namespace std::literals::chrono_literals;

ThreadedInDataHandler::ThreadedInDataHandler(StreamMgr &mgr)
    : InDataHandler(mgr) {

  DEBUGTRACE_ENTER;

  // Initialize thread pool, if not already done
  getPool();
}

bool ThreadedInDataHandler::inCallback(const DaqData &daqdata) {
  /* DEBUGTRACE_ENTER; */

  if (!_lastCallbackResult) {
    return false;
  }
  dataqueue.push(daqdata);

  auto &pool = getPool();

  if (!_thread_running && (!_stopThread) && _lastCallbackResult) {
    pool.push_task(&ThreadedInDataHandler::threadFcn, this);
  }

  return _lastCallbackResult;
}

ThreadedInDataHandler::~ThreadedInDataHandler() {

  DEBUGTRACE_ENTER;
  _stopThread = true;

  // Then wait in steps for the thread to stop running.
  while (_thread_running) {
    std::this_thread::sleep_for(10us);
  }
}

void ThreadedInDataHandler::threadFcn() {

  /* DEBUGTRACE_ENTER; */
  _thread_running = true;

  DaqData d{};

  if (dataqueue.pop(d) && !_stopThread) {

    // Call inCallback_threaded
    if (inCallback_threaded(d) == false) {
      _lastCallbackResult = false;
      _thread_running = false;
      return;
    }
  }
  _lastCallbackResult = true;
  _thread_running = false;
}
